package cn.fw.dalaran.domain.dto;

import cn.fw.dalaran.domain.enums.AccountStatusEnum;
import cn.fw.dalaran.domain.enums.PlatformEnum;
import lombok.Data;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.util.Objects;

/**
 * @author kurisu
 * @date 2021-11-16 11:14
 * @description :
 */
@Data
public class AccountDTO {
    /**
     * 账号id
     */
    private Long id;
    /**
     * 账号
     */
    @NotBlank(message = "账号不能为空")
    private String account;
    /**
     * 账户名
     */
    private String accountName;
    /**
     * 回播搜索关键词
     */
    private String playbackSearchKey;
    /**
     * 认证状态(0:未认证, 1:已认证)
     */
    private Integer affirm;
    /**
     * 平台类型
     */
    @NotNull(message = "平台类型不能为空")
    private Integer type;
    /**
     * 账号类型
     */
    @NotNull(message = "账号类型不能为空")
    private Integer accountType;
    /**
     * 账号状态(-1:已离职, 0:未上班, 1:上班中)
     */
    private Integer accountStatus;
    /**
     * 用户id
     */
    @NotNull(message = "用户id不能为空")
    private Long userId;
    /**
     * 用户名称
     */
    @NotBlank(message = "用户名称不能为空")
    private String userName;
    /**
     * 门店id
     */
    @NotNull(message = "门店id不能为空")
    private Long shopId;
    /**
     * 门店名称
     */
    @NotBlank(message = "门店名称不能为空")
    private String shopName;
    /**
     * 集团id
     */
    //@LoginContextField(LoginContextField.Name.GROUP_ID)
    private Long groupId;

    public Boolean getAffirm() {
        if (Objects.isNull(affirm)) {
            return null;
        }
        return Objects.equals(affirm, 1);
    }

    public PlatformEnum getType() {
        if (Objects.isNull(type)) {
            return null;
        }
        return PlatformEnum.ofValue(type);
    }

    public AccountStatusEnum getAccountStatus() {
        if (Objects.isNull(accountStatus)) {
            return null;
        }
        return AccountStatusEnum.ofValue(accountStatus);
    }
}
